// oper.h - Klase operatora.

#ifndef _oper_h_
#define _oper_h_

#include "izraz.h"
#include <cmath>
using namespace std;

namespace Izrazi {
  class Oper: public Izraz { // APSTRAKTNA KLASA BINARNIH OPERATORA.
  protected:
    Izraz *a, *b;                          // Operandi.
    void pisi(ostream& it, char op) const  // Pisanje.
      { it << '(' << *a << op << *b << ')'; }
  private:
    void kopiraj(const Oper& op)           // Kopiranje u objekat.
      { a = op.a->kopija(); b = op.b->kopija(); }
    void brisi() { delete a; delete b; }   // Oslobadanje memorije.
  public:
    Oper(const Izraz& x, const Izraz& y)   // Inicijalizacija izrazima.
      { a = x.kopija(); b = y.kopija(); }
    Oper(const Oper& op) { kopiraj(op); }  // Konstruktor kopije.
    ~Oper() { brisi(); }                   // Destruktor.
    Oper& operator=(const Oper& y) {       // Dodela vrednosti.
      if (this != &y) { brisi(); kopiraj(y); }
      return *this;
    }
  }; // class Oper

  class Zbir: public Oper {                              // KLASA ZBIROVA.
    void pisi(ostream& it) const { Oper::pisi(it,'+'); } // Pisanje.
  public:
    Zbir(const Izraz& x, const Izraz& y): Oper(x, y) {}  // Stvaranje.
    double vredn() const { return a->vredn() + b->vredn(); } // Vrednost.
    Zbir* kopija() const { return new Zbir(*this); }     // Kopiranje.
  }; // class Zbir

  class Razl: public Oper {                              // KLASA RAZLIKA.
    void pisi(ostream& it) const { Oper::pisi(it,'-'); } // Pisanje.
  public:
    Razl(const Izraz& x, const Izraz& y): Oper(x, y) {}  // Stvaranje.
    double vredn() const { return a->vredn() - b->vredn(); } // Vrednost.
    Razl* kopija() const { return new Razl(*this); }     // Kopiranje.
  }; // class Razl

  class Proizv: public Oper {                            // KL. PROIZVODA.
    void pisi(ostream& it) const { Oper::pisi(it,'*'); } // Pisanje.
  public:
    Proizv(const Izraz& x, const Izraz& y): Oper(x,y) {} // Stvaranje.
    double vredn() const { return a->vredn() * b->vredn(); } // Vrednost.
    Proizv* kopija() const { return new Proizv(*this); } // Kopiranje.
  }; // class Proizv

  class Kolic: public Oper {                             // KL. KOLICNIKA.
    void pisi(ostream& it) const { Oper::pisi(it,'/'); } // Pisanje.
  public:
    Kolic(const Izraz& x, const Izraz& y): Oper(x, y) {} // Stvaranje.
    double vredn() const { return a->vredn() / b->vredn(); } // Vrednost.
    Kolic* kopija() const { return new Kolic(*this); }   // Kopiranje.
  }; // class Kolic
 
  class Step: public Oper {                              // KLASA STEPENA.
    void pisi(ostream& it) const { Oper::pisi(it,'^'); } // Pisanje.
  public:
    Step(const Izraz& x, const Izraz& y): Oper(x, y) {}  // Stvaranje.
    double vredn() const                                 // Vrednost.
      { return pow(a->vredn(), b->vredn()); }
    Step* kopija() const { return new Step(*this); }     // Kopiranje.
  }; // class Step

  // POMOCNE FUNKCIJE ZA OMOGUCAVANJE PISANJA CITLJIVIH IZRAZA.
  inline Zbir   operator+ (const Izraz& a, const Izraz& b) // a +  b
    { return Zbir  (a, b); }
  inline Razl   operator- (const Izraz& a, const Izraz& b) // a -  b
    { return Razl  (a, b); }
  inline Proizv operator* (const Izraz& a, const Izraz& b) // a *  b
    { return Proizv(a, b); }
  inline Kolic  operator/ (const Izraz& a, const Izraz& b) // a /  b
    { return Kolic (a, b); }
  inline Step   operator^ (const Izraz& a, const Izraz& b) // a ^  b
    { return Step  (a, b); }
  inline bool   operator==(const Izraz& a, const Izraz& b) // a == b
    { return a.vredn() == b.vredn(); }
  inline bool   operator!=(const Izraz& a, const Izraz& b) // a != b
    { return a.vredn() != b.vredn(); }
  inline bool   operator< (const Izraz& a, const Izraz& b) // a <  b
    { return a.vredn() <  b.vredn(); }
  inline bool   operator<=(const Izraz& a, const Izraz& b) // a <= b
    { return a.vredn() <= b.vredn(); }
  inline bool   operator> (const Izraz& a, const Izraz& b) // a >  b
    { return a.vredn() >  b.vredn(); }
  inline bool   operator>=(const Izraz& a, const Izraz& b) // a >= b
    { return a.vredn() >= b.vredn(); }

} // namespace Izrazi;

#endif
