// prom.h - Klasa promenljivih.

#ifndef _prom_h_
#define _prom_h_

#include "izraz.h"
#include <cstring>
using namespace std;

namespace Izrazi {
  class Prom: public Izraz {
    struct Elem {                     // Element liste svih promenljivih:
      char* ime;                      //  - ime,
      double vr;                      //  - vrednost,
      int brKor;                      //  - broj koricenja,
      Elem *pret, *sled;              //  - prethodni i sledeci u listi,
      Elem(const char* i, double v,   //  - konstruktor,
            Elem* p=0, Elem* s=0) {
        ime = new char [strlen(i)+1]; strcpy(ime, i);
        vr = v; pret = p; sled = s; brKor = 1;
      }
      ~Elem() { delete [] ime; }      //  - destruktor.
    }; // struct Elem

    static Elem* prvi;                // Pocetak liste svih promenljivih.
    Elem* ovaj;                       // Element pridruen ovom objektu.
    void pravi(const char* ime, double vr); // Stvaranje objekta.
    void pisi(ostream& it) const { it << ovaj->ime; } // Pisanje.
  public:                                     // Inicijalizacija:
    Prom(const char* ime, double vr=0)        // - imenom i brojem,
      { pravi(ime, vr); }
    Prom(const char* ime, const Izraz& i)     // - imenom i izrazom,
      { pravi(ime, i.vredn()); }
    Prom(const Prom& p)                       // - kopijom promenljive.
      { ovaj = p.ovaj; ovaj->brKor++; }
    ~Prom() {                                 // Unitavanje.
      if (--ovaj->brKor == 0) {
        (ovaj->pret ? ovaj->pret->sled : prvi) = ovaj->sled;
        if (ovaj->sled) ovaj->sled->pret = ovaj->pret;
        delete ovaj;
      }
    }                                         // Dodela vrednosti:
    Prom& operator=(double v)                 // - realnog broja,
      { ovaj->vr = v; return *this; }
    Prom& operator=(const Izraz& i)           // - izraza,
      { ovaj->vr = i.vredn(); return *this; }
    Prom& operator=(const Prom& p)            // - druge promenljive.
      { ovaj->vr = p.vredn(); return *this; }
    double vredn() const { return ovaj->vr; } // Vrednost promenljive.
    Prom* kopija() const { return new Prom(*this); } // Kopiranje.
    friend istream& operator>>(istream& ut, Prom& p) // Citanje.
      { return ut >> p.ovaj->vr; }
    static void pisiSve(ostream& it);         // Spisak svih promenljivih.
  }; // class Prom
} // namespace Izrazi

#endif
