// racbroj.h - Klasa racionalnih brojeva.

#ifndef _racbroj_h_
#define _racbroj_h_

#include <iostream>
#include <cstdlib>
using namespace std;

namespace Aritmetika {
  class GRacBroj {}; // Klasa greaka: Imenilac jednak nuli.

  class RacBroj {
    typedef const RacBroj CRB;          // Nepromenljiv racionalan broj.
    long x, y;                          // Brojilac i delilac.
    static long nzd(long a, long b);    // Najveci zajednicki delilac.
  public:
    RacBroj(long a = 0, long b = 1) {   // Inicijalizacija.
      x = a; y = b;
      if (y == 0) throw GRacBroj();
      if (y < 0) { x = -x; y = -y; }
      long z = nzd(labs(x), y); x /= z; y /= z;
    }
    RacBroj operator-() const { return RacBroj(-x, y); }       // -r
    RacBroj operator+() const { return *this; }                // +r
    RacBroj operator!() const { return RacBroj(y, x); }        // 1 / r

    friend RacBroj operator+(CRB& r, CRB& s)                   // r + s
      { return RacBroj(r.x*s.y+s.x*r.y, r.y*s.y); }
    friend RacBroj operator-(CRB& r, CRB& s)                   // r - s
      { return RacBroj(r.x*s.y-s.x*r.y, r.y*s.y); }
    friend RacBroj operator*(CRB& r, CRB& s)                   // r * s
      { return RacBroj(r.x*s.x, r.y*s.y); }
    friend RacBroj operator/(CRB& r, CRB& s)                   // r / s
      { return RacBroj(r.x*s.y, r.y*s.x); }

    RacBroj& operator+=(CRB& s) { return *this = *this + s; }  // r += s
    RacBroj& operator-=(CRB& s) { return *this = *this - s; }  // r -= s
    RacBroj& operator*=(CRB& s) { return *this = *this * s; }  // r *= s
    RacBroj& operator/=(CRB& s) { return *this = *this / s; }  // r /= s
 
    RacBroj& operator++() { return *this += 1; }                      // ++r
    RacBroj& operator--() { return *this -= 1; }                      // --r
    RacBroj  operator++(int) { RacBroj a(*this); ++*this; return a; } // r++
    RacBroj  operator--(int) { RacBroj a(*this); --*this; return a; } // r--

    friend bool operator==(CRB& r, CRB& s)                     // r == s
      { return r.x==s.x && r.y==s.y; }
    friend bool operator!=(CRB& r, CRB& s)                     // r != s
      { return r.x!=s.x || r.y!=s.y; }
    friend bool operator< (CRB& r, CRB& s)                     // r <  s
      { return r.x* s.y <  r.y* s.x; }
    friend bool operator<=(CRB& r, CRB& s)                     // r <= s
      { return r.x* s.y <= r.y* s.x; }
    friend bool operator> (CRB& r, CRB& s)                     // r >  s
      { return r.x* s.y >  r.y* s.x; }
    friend bool operator>=(CRB& r, CRB& s)                     // r >= s
      { return r.x* s.y >= r.y* s.x; }

    friend istream& operator>>(istream& ut, RacBroj& r);       // Citanje.
    friend ostream& operator<<(ostream& it, CRB& r);           // Pisanje.
  };
}

#endif
