// skup2.h - Klasa skupova apstraktnih objekata.

#ifndef _skup2_h_
#define _skup2_h_

#include "objekat.h"

class Skup {
  struct Elem {                                    // Element skupa:
    Obj* obj;                                      // - sadraj,
    Elem* sled;                                    // - sledeci,
    Elem(Obj* ob, Elem* sl=0) { obj=ob; sled=sl; } // - konstruktor,
    ~Elem() { delete obj; }                        // - destruktor.
  };

  Elem *prvi, *posl;                               // Elementi skupa.
  void dodaj(const Obj& obj) {                     // Dodavanje objekta.
    Elem* novi = new Elem(obj.kopija());
    posl = (posl ? posl->sled : prvi) = novi;
  }
  void kopiraj(const Skup& s);                     // Kopiranje skupa.
  void brisi();                                    // Unitavanje skupa.
public:
  Skup() { prvi = posl = 0; }                      // Prazan skup.
  Skup(const Obj& obj)                             // Konverzija.
    { prvi = posl = new Elem(obj.kopija()); }
  Skup(const Skup& s) { kopiraj(s); }              // Kopija.
  ~Skup() { brisi(); }                             // Unitavanje.
  Skup& operator=(const Skup& s) {                 // Dodela.
    if (this != &s) { brisi(); kopiraj(s); }
    return *this;
  }
  bool operator%(const Obj& obj) const;              // Element skupa?
  friend Skup operator+(const Skup& s1, const Skup& s2)         // Unija.
    { return Skup(s1) += s2; }
  friend Skup operator-(const Skup& s1, const Skup& s2);        // Razlika.
  friend Skup operator*(const Skup& s1, const Skup& s2);        // Presek.
  Skup& operator+=(const Skup& s);                              // Unija.
  Skup& operator-=(const Skup& s) { return *this = *this - s; } // Razlika.
  Skup& operator*=(const Skup& s) { return *this = *this * s; } // Presek.
  friend ostream& operator<<(ostream& it, const Skup& s);       // Pisanje.
};

#endif
